import static org.junit.jupiter.api.Assertions.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import greenfoot.junitUtils.jupiter.runner.GreenfootRunner;
import greenfoot.junitUtils.*;

@ExtendWith(GreenfootRunner.class)
class BallTest {
    MyWorld world;
    Ball ball;

    @BeforeEach
    void setUp() throws Exception { 
        // GIVEN: The local variables one world and one ball 
        world = new MyWorld();
        ball = world.getObjects(Ball.class).get(0);
    }    


    @Test
    void testOneAct() throws Exception {
        // WHEN : The action to test is one act cycles
        WorldCreator.runOnce(world);    
    
        // THEN : The asserts to verify is the ball position
        assertEquals(2, ball.getX());
        assertEquals(2, ball.getY());
    }  
 
    @Test
    void testOneActBis() throws Exception {
        // WHEN : The action to test is one act cycles
        ball.act();    
    
        // THEN : The asserts to verify is the ball position
        assertEquals(2, ball.getX());
        assertEquals(2, ball.getY());
    }  
 
    @Test
    void testOneActBisBis() throws Exception {
        // WHEN : The action to test is one act cycles
        WorldCreator.runOnce();    
    
        // THEN : The asserts to verify is the ball position
        assertEquals(2, ball.getX());
        assertEquals(2, ball.getY());
    }  
 
    @Test
    void testTwoAct() throws Exception {
        // WHEN : The action to test is two act cycles
        WorldCreator.runOnce(world);
        WorldCreator.runOnce(world);  

        // THEN : The asserts to verify is the ball position
        assertEquals(2, ball.getX());
        assertEquals(3, ball.getY());
    }   

    @Test
    void testTwoActBis() throws Exception {
        // WHEN : The action to test is two act cycles
        ball.act();    
        ball.act();     

        // THEN : The asserts to verify is the ball position
        assertEquals(2, ball.getX());
        assertEquals(3, ball.getY());
    }   

    @Test
    void testTwoActBisBis() throws Exception {
        // WHEN : The action to test is two act cycles
        WorldCreator.runOnce();
        WorldCreator.runOnce();  

        // THEN : The asserts to verify is the ball position
        assertEquals(2, ball.getX());
        assertEquals(3, ball.getY());
    }   


    @Test
    void testClickedBall() throws Exception {
        // WHEN : The action to test is to click the ball
        EventDispatch.mouseClicked(2, 1);
        WorldCreator.runOnce(world);   
     
        // THEN : The asserts to verify is the ball position
        assertEquals(3, ball.getX());
        assertEquals(1, ball.getY());
    }   
    
    @Test
    void testClickedBallBis() throws Exception {
        // WHEN : The action to test is to click the ball
        EventDispatch.mouseClicked(2, 1);
        ball.act();     
     
        // THEN : The asserts to verify is the ball position
        assertEquals(3, ball.getX());
        assertEquals(1, ball.getY());
    } 
  
    @Test
    void testClickedBallBisBis() throws Exception {
        // WHEN : The action to test is to click the ball
        EventDispatch.mouseClicked(2, 1);
        WorldCreator.runOnce();   
     
        // THEN : The asserts to verify is the ball position
        assertEquals(3, ball.getX());
        assertEquals(1, ball.getY());
    }   
    
    @Test
    void testClickedBallAfterFallOut() throws Exception {        
        // WHEN : The action to test is to click the ball
        WorldCreator.runOnce(world);
        EventDispatch.mouseClicked(2, 2);
        WorldCreator.runOnce(world);

        // THEN : The asserts to verify is the ball position
        assertEquals(3, ball.getX());
        assertEquals(2, ball.getY());
    }   

    @Test
    void testClickedBallAfterFallOutBis() throws Exception {        
        // WHEN : The action to test is to click the ball
        ball.act();     
        EventDispatch.mouseClicked(2, 2);
        ball.act();     

        // THEN : The asserts to verify is the ball position
        assertEquals(3, ball.getX());
        assertEquals(2, ball.getY());
    }   

    @Test
    void testClickedBallAfterFallOutBisBis() throws Exception {        
        // WHEN : The action to test is to click the ball
        WorldCreator.runOnce(world);
        EventDispatch.mouseClicked(2, 2);
        WorldCreator.runOnce();

        // THEN : The asserts to verify is the ball position
        assertEquals(3, ball.getX());
        assertEquals(2, ball.getY());
    }   

    @Test
    void testImageChanged() throws Exception {        
        // WHEN : The action to test is image changed
        Random.set(19);
        WorldCreator.runOnce(world);

        // THEN : The asserts to verify that the image is changed
        assertEquals("steel-ball.png",
                     ball.getImage().getImageFileName());
    } 
  
    @Test
    void testImageChangedBis() throws Exception {        
        // WHEN : The action to test is image changed
        Random.set(19);
        ball.act();     

        // THEN : The asserts to verify that the image is changed
        assertEquals("steel-ball.png", 
                     ball.getImage().getImageFileName());
    } 
  
    @Test
    void testImageChangedBisBis() throws Exception {        
        // WHEN : The action to test is image changed
        Random.set(19);
        WorldCreator.runOnce();

        // THEN : The asserts to verify that the image is changed
        assertEquals("steel-ball.png",
                     ball.getImage().getImageFileName());
    } 

    @Test
    void testImageNotChanged() throws Exception {        
        // WHEN : The action to test is image changed
        Random.set(20);
        WorldCreator.runOnce(world);

        // THEN : The asserts to verify is that it is the same image
        assertEquals("gold-ball.png", ball.getImage().getImageFileName());
    }

    @Test
    void testImageNotChangedBis() throws Exception {        
        // WHEN : The action to test is image changed
        Random.set(20);
        ball.act();     

        // THEN : The asserts to verify is that it is the same image
        assertEquals("gold-ball.png", ball.getImage().getImageFileName());
    }   

    @Test
    void testImageNotChangedBisBis() throws Exception {        
        // WHEN : The action to test is image changed
        Random.set(20);
        WorldCreator.runOnce();

        // THEN : The asserts to verify is that it is the same image
        assertEquals("gold-ball.png", ball.getImage().getImageFileName());
    }

    @Test
    void testImageChangedAftertwoActCycles() throws Exception {  
        // GIVEN:
        Random.set(20);
        Random.set(20);
        WorldCreator.runOnce(world);

        // WHEN : The action to test is image changed
        Random.set(19);
        ball.act();

        // THEN : The asserts to verify that the image is changed
        assertEquals("steel-ball.png", ball.getImage().getImageFileName());
    } 

    @Test
    void testImageChangedAftertwoActCyclesBis() throws Exception {  
        // GIVEN:
        Random.set(20);
        Random.set(20);
        WorldCreator.runOnce();

        // WHEN : The action to test is image changed
        Random.set(19);
        ball.act();

        // THEN : The asserts to verify that the image is changed
        assertEquals("steel-ball.png", ball.getImage().getImageFileName());
    } 

}
